<?php

declare(strict_types=1);

namespace Drupal\sm_transport_drupal_sql\Transport;

use Symfony\Component\Messenger\Envelope;
use Symfony\Component\Messenger\Exception\TransportException;
use Symfony\Component\Messenger\Stamp\DelayStamp;
use Symfony\Component\Messenger\Stamp\TransportMessageIdStamp;
use Symfony\Component\Messenger\Transport\Sender\SenderInterface;
use Symfony\Component\Messenger\Transport\Serialization\PhpSerializer;
use Symfony\Component\Messenger\Transport\Serialization\SerializerInterface;

/**
 * A sender for the Drupal SQL transport.
 */
class DrupalSQLSender implements SenderInterface {

  /**
   * A serializer to encode the message data.
   *
   * @var \Symfony\Component\Messenger\Transport\Serialization\SerializerInterface
   */
  private SerializerInterface $serializer;

  /**
   * DrupalSQLSender constructor.
   *
   * @param \Drupal\sm_transport_drupal_sql\Transport\DrupalSQLConnection $connection
   *   The DrupalSQLConnection object.
   * @param \Symfony\Component\Messenger\Transport\Serialization\SerializerInterface|null $serializer
   *   The serializer to use.
   */
  public function __construct(
    private readonly DrupalSQLConnection $connection,
    ?SerializerInterface $serializer = null,
  ) {
    $this->serializer = $serializer ?? new PhpSerializer();
  }

  /**
   * {@inheritdoc}
   */
  public function send(Envelope $envelope): Envelope {
    $encodedMessage = $this->serializer->encode($envelope);

    /** @var DelayStamp|null $delayStamp */
    $delayStamp = $envelope->last(DelayStamp::class);
    $delay = null !== $delayStamp ? $delayStamp->getDelay() : 0;

    try {
      $id = $this->connection->send($encodedMessage['body'], $encodedMessage['headers'] ?? [], $delay);
    } catch (\Exception $exception) {
      throw new TransportException($exception->getMessage(), 0, $exception);
    }

    return $envelope->with(new TransportMessageIdStamp($id));
  }

}
