<?php

declare(strict_types=1);

namespace Drupal\sm_transport_drupal_sql\Transport;

use Drupal\Core\Database\Connection;
use Symfony\Component\Messenger\Transport\Serialization\SerializerInterface;
use Symfony\Component\Messenger\Transport\TransportFactoryInterface;
use Symfony\Component\Messenger\Transport\TransportInterface;

/**
 * Messenger transport factory for Drupal SQL.
 */
class DrupalSQLTransportFactory implements TransportFactoryInterface {

  /**
   * DrupalSQLTransportFactory constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   */
  public function __construct(
    private readonly Connection $connection,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public function createTransport(
    string $dsn,
    array $options,
    SerializerInterface $serializer,
  ): TransportInterface {
    $configuration = DrupalSQLConnection::buildConfiguration($dsn, $options);

    return new DrupalSQLTransport($this->connection, $serializer, $configuration);
  }

  /**
   * Check if the factory supports the given DSN.
   */
  public function supports(string $dsn, array $options): bool {
    return str_starts_with($dsn, 'drupal-db://');
  }

}
